package arzumify.polyenergy.api;

// Receives energy from EnergyProviders. Can be implemented by anything able to tick and receive energy!
public interface EnergyReceiver extends EnergyBlock {
    /**
     * When called by a provider, indicates that the provider exists and is ready to have energy extracted from.
     * Do not attempt to call {@link EnergyProvider#extract(long, EnergyReceiver)} on any provider until this method is called.
     * We recommend using a Set to store the providers that are ready to provide energy.
     *
     * @param provider The provider that exists and is ready to provide energy to this receiver.
     */
    void ready(EnergyProvider provider);

    /**
     * When called by the provider, indicates that the provider no longer exists or is no longer able to provide energy.
     * You do not need to call {@link EnergyProvider#unready(EnergyReceiver)} on the provider.
     *
     * @param provider The provider that no longer exists.
     */
    void unready(EnergyProvider provider);
}
